@@ -13,7 +13,7 @@ module Msf::DBManager::Import::Appscan
 
   def import_appscan_xml(args={}, &block)
     bl = validate_ips(args[:blacklist]) ? args[:blacklist].split : []
-    wspace = args[:wspace] || workspace
+    wspace = Msf::Util::DBManager.process_opts_workspace(args, framework).name
     if Rex::Parser.nokogiri_loaded
       parser = "Nokogiri v#{::Nokogiri::VERSION}"
       noko_args = args.dup
@@ -30,4 +30,4 @@ module Msf::DBManager::Import::Appscan
       raise Msf::DBImportError.new("Could not import due to missing Nokogiri parser. Try 'gem install nokogiri'.")
     end
   end
-end
\ No newline at end of file
+end
