@@ -446,8 +446,10 @@ module Exploit::Remote::HttpClient
       opts['vhost'] = opts['vhost'] || opts['rhost'] || self.vhost()
       opts['rport'] = datastore['RPORT']
 
-      opts['ssl'] = ssl
+      opts['SSL'] = ssl
     else
+      disconnect
+
       opts['redirect_uri'] = location
       opts['uri'] = location.path
       opts['rhost'] = location.host
@@ -455,9 +457,9 @@ module Exploit::Remote::HttpClient
       opts['rport'] = location.port
 
       if location.scheme == 'https'
-        opts['ssl'] = true
+        opts['SSL'] = true
       else
-        opts['ssl'] = false
+        opts['SSL'] = false
       end
     end
   end
