@@ -24,6 +24,10 @@ module Exploit::Remote::RDP
         OptAddress.new('RDP_CLIENT_IP', [ true, 'The client IPv4 address to report during connect', '192.168.0.100']),
         Opt::RPORT(3389)
       ], Msf::Exploit::Remote::RDP)
+      register_advanced_options(
+      [
+        OptInt.new('RDP_TLS_SECURITY_LEVEL', [ true, 'Change default TLS security level. "0" (default) means everything is permitted. "1" rejects very weak parameters and "2" is even stricter.', 0 ])
+      ], Msf::Exploit::Remote::RDP)
   end
 
 
@@ -64,19 +68,94 @@ module Exploit::Remote::RDP
       5 => "HYBRID_REQUIRED_BY_SERVER",
       6 => "SSL_WITH_USER_AUTH_REQUIRED_BY_SERVER"
     }
+
+    REDIRECTION_SUPPORTED = 0x1
+    REDIRECTION_VERSION3  = 0x2 << 2
+    REDIRECTION_VERSION4  = 0x3 << 2
+    REDIRECTION_VERSION5  = 0x4 << 2
+
+    ENCRYPTION_40BIT  = 0x01
+    ENCRYPTION_128BIT = 0x02
+    ENCRYPTION_56BIT  = 0x08
+    ENCRYPTION_FIPS   = 0x10
+
+    LICENSE_REQUEST            = 0x01
+    LICENSE_PLATFORM_CHALLENGE = 0x02
+    LICENSE_NEW_LICENSE        = 0x03
+    LICENSE_UPGRADE_LICENSE    = 0x04
+    LICENSE_LICENSE_INFO       = 0x12
+    LICENSE_NEW_LICENSE_REQ    = 0x13
+    LICENSE_PLATFORM_CHAL_RESP = 0x15
+    LICENSE_ERROR_ALERT        = 0xff
+
+    LICENSE_TAGS = {
+      0x01 => "LICENSE_REQUEST",
+      0x02 => "LICENSE_PLATFORM_CHALLENGE",
+      0x03 => "LICENSE_NEW_LICENSE",
+      0x04 => "LICENSE_UPGRADE_LICENSE",
+      0x12 => "LICENSE_LICENSE_INFO",
+      0x13 => "LICENSE_NEW_LICENSE_REQ",
+      0x15 => "LICENSE_PLATFORM_CHAL_RESP",
+      0xff => "LICENSE_ERROR_ALERT"
+    }
+
+    LICENSE_ERR_INVALID_SCOPE     = 0x04
+    LICENSE_ERR_NO_LICENSE_SERVER = 0x06
+    LICENSE_ERR_LICENSE_ISSUED    = 0x07
+    LICENSE_ERR_INVALID_CLIENT    = 0x08
+    LICENSE_ERR_INVALID_PRODUCTID = 0x0b
+    LICENSE_ERR_INVALID_MSG_LEN   = 0x0c
+
+    LICENSE_ERRS = {
+      0x04 => "INVALID_SCOPE",
+      0x06 => "NO_LICENSE_SERVER",
+      0x07 => "LICENSE_ISSUED",
+      0x08 => "INVALID_CLIENT",
+      0x0b => "INVALID_PRODUCTID",
+      0x0c => "INVALID_MSG_LEN"
+    }
+
+    CHAN_INITIALIZED               = 0x80000000
+    CHAN_ENCRYPT_RDP               = 0x40000000
+    CHAN_ENCRYPT_SC                = 0x20000000
+    CHAN_ENCRYPT_CS                = 0x10000000
+    CHAN_PRI_HIGH                  = 0x08000000
+    CHAN_PRI_MED                   = 0x04000000
+    CHAN_PRI_LOW                   = 0x02000000
+    CHAN_COMPRESS_RDP              = 0x00800000
+    CHAN_COMPRESS                  = 0x00400000
+    CHAN_SHOW_PROTOCOL             = 0x00200000
+    CHAN_REMOTE_CONTROL_PERSISTENT = 0x00100000
+
+    CHAN_FLAG_FIRST         = 0x01
+    CHAN_FLAG_LAST          = 0x02
+    CHAN_FLAG_SHOW_PROTOCOL = 0x10
+
+    RDPDR_CTYP_CORE = 0x4472
+
+    PAKID_CORE_SERVER_ANNOUNCE     = 0x496e
+    PAKID_CORE_SERVER_CAPABILITY   = 0x5350
+    PAKID_CORE_CLIENTID_CONFIRM    = 0x4343
+    PAKID_CORE_CLIENT_NAME         = 0x434e
+    PAKID_CORE_DEVICELIST_ANNOUNCE = 0x4441
   end
 
+  def rdp_connect
+    self.rdp_sock = connect(false)
+    self.rdp_sock.setsockopt(::Socket::IPPROTO_TCP, ::Socket::TCP_NODELAY, 1)
+  end
 
-  #
-  # Protocol Communications
-  #
+  def rdp_disconnect
+    disconnect(self.rdp_sock)
+    self.rdp_sock = nil
+  end
 
   def rdp_send(data)
-    sock.put(data)
+    self.rdp_sock.put(data)
   end
 
-  def rdp_recv
-    res = sock.get_once(-1, 5)
+  def rdp_recv(length = -1, timeout = 5)
+    res = self.rdp_sock.get_once(length, timeout)
     raise RdpCommunicationError unless res # nil due to a timeout
 
     res
@@ -95,14 +174,14 @@ module Exploit::Remote::RDP
   #
   # @return [Boolean] Is service RDP
   # @return [Hash] Version information
-  def rdp_fingerprint(nsock)
+  def rdp_fingerprint
     peer_info = {}
     # warning: if rdp_check_protocol starts handling NLA, this will need to be updated
     is_rdp, server_selected_proto = rdp_check_protocol(RDPConstants::PROTOCOL_SSL | RDPConstants::PROTOCOL_HYBRID | RDPConstants::PROTOCOL_HYBRID_EX)
     return false, nil unless is_rdp
     return true, peer_info unless [RDPConstants::PROTOCOL_HYBRID, RDPConstants::PROTOCOL_HYBRID_EX].include? server_selected_proto
 
-    swap_sock_plain_to_ssl(nsock)
+    swap_sock_plain_to_ssl
     ntlm_negotiate_blob = ''  # see: https://fadedlab.wordpress.com/2019/06/13/using-nmap-to-extract-windows-info-from-rdp/
     ntlm_negotiate_blob << "\x30\x37\xa0\x03\x02\x01\x60\xa1\x30\x30\x2e\x30\x2c\xa0\x2a\x04\x28"
     ntlm_negotiate_blob << "\x4e\x54\x4c\x4d\x53\x53\x50\x00"  #  Identifier - NTLMSSP
@@ -131,6 +210,40 @@ module Exploit::Remote::RDP
     return is_rdp, peer_info
   end
 
+  def rdp_dispatch_loop
+    while rdp_sock do
+      rdp_handle_packet(rdp_recv)
+    end
+  end
+
+  def rdp_create_channel_msg(chan_user_id, chan_id, data, flags = 3, data_length = nil)
+    data_length ||= data.length
+
+    pdu = [
+      [25 << 2].pack('C'), # MCS send data request structure, choice 25
+      [self.rdp_user_id, chan_id].pack('S>S>'), # MCS send data request structure, choice 25
+      "\x70", # Wut (security header)
+      per_data(
+        [data_length].pack('<L'),
+        [flags].pack('<L'),
+        data
+      )
+    ].join('')
+
+    build_data_tpdu(pdu)
+  end
+
+  def rdp_send_channel(chan_user_id, chan_id, data, flags = 3, data_length = nil)
+    tpkt = rdp_create_channel_msg(chan_user_id, chan_id, data, flags, data_length)
+    rdp_send(tpkt)
+  end
+
+  def rdp_terminate
+    body = "\x21\x80" # user requested disconnect provider ultimatum
+
+    rdp_send(build_data_tpdu(body))
+  end
+
   # Connect and detect security protocol
   #
   # Note: NLA is detected but not supported yet
@@ -182,8 +295,8 @@ module Exploit::Remote::RDP
       if ["SSL_NOT_ALLOWED_BY_SERVER", "Negotiation Response packet too short."].include? err_msg
         # This happens if the server is configured to ONLY permit RDP Security
         vprint_status("Attempting to connect using Standard RDP security")
-        disconnect
-        connect
+        rdp_disconnect
+        rdp_connect
         res = rdp_send_recv(pdu_negotiation_request(@user_name, RDPConstants::PROTOCOL_RDP))
 
         if res
@@ -208,16 +321,14 @@ module Exploit::Remote::RDP
   # Negotiate security protocol and begin session building
   #
   # @return [Boolean] success
-  def rdp_negotiate_security(nsock, req_proto = RDPConstants::PROTOCOL_SSL)
+  def rdp_negotiate_security(channels, req_proto = RDPConstants::PROTOCOL_SSL)
     if req_proto == RDPConstants::PROTOCOL_SSL
-      swap_sock_plain_to_ssl(nsock)
-      res = rdp_send_recv(pdu_connect_initial(req_proto, @computer_name))
-
+      swap_sock_plain_to_ssl
+      res = rdp_send_recv(pdu_connect_initial(channels, req_proto, @computer_name))
     elsif req_proto == RDPConstants::PROTOCOL_RDP
-      res = rdp_send_recv(pdu_connect_initial(req_proto, @computer_name))
+      res = rdp_send_recv(pdu_connect_initial(channels, req_proto, @computer_name))
       rsmod, rsexp, _rsran, server_rand, bitlen = rdp_parse_connect_response(res)
-
-    elsif [RDPConstants::PROTOCOL_HYBRID, RDPConstants::PROTOCOL_HYBRID_EX].include? req_proto
+    elsif [RDPConstants::PROTOCOL_HYBRID, RDPConstants::PROTOCOL_HYBRID_EX].include?(req_proto)
       vprint_status("NLA Security protocol unsupported at this time.")
       return false
     else
@@ -230,11 +341,11 @@ module Exploit::Remote::RDP
     rdp_send(pdu_erect_domain_request)
     res = rdp_send_recv(pdu_attach_user_request)
 
-    user1 = res[9, 2].unpack("n").first
+    self.rdp_user_id = res[9, 2].unpack("n").first
 
     # send channel requests
     [1009, 1003, 1004, 1005, 1006, 1007, 1008].each do |chan|
-      rdp_send_recv(pdu_channel_request(user1, chan))
+      rdp_send_recv(pdu_channel_join_request(self.rdp_user_id, chan))
     end
 
     if req_proto == RDPConstants::PROTOCOL_RDP
@@ -258,11 +369,80 @@ module Exploit::Remote::RDP
     return true
   end
 
+  def rdp_generate_license_keys(data)
+    client_random = ''
+    32.times { client_random << rand(0..255) }
+    premaster_secret = ''
+    32.times { premaster_secret << rand(0..255) }
+    server_random = data[0..31]
+
+    master_secret = rdp_salted_hash48(premaster_secret, "A", client_random, server_random)
+    key_block = rdp_salted_hash48(master_secret, "A", client_random, server_random)
+
+    license_sign_key = key_block[0..15]
+    license_key = rdp_salted_hash16(key_block[16..31], client_random, server_random)
+
+    return client_random, license_key, license_sign_key
+  end
+
+  # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpele/e17772e9-9642-4bb6-a2bc-82875dd6da7c
+  # Server License Request - 2.2.2.1
+  def rdp_handle_license_request(data)
+    # Note: license_key is currently unused
+    client_random, license_key, license_sign_key = rdp_generate_license_keys(data)
+
+    # We're not really decrypting the license from the server, but it should be good enough
+    vprint_status("Sending new license request PDU")
+    new_license_request = pdu_new_license_request(client_random, @user_name, @computer_name)
+    pkt = rdp_build_pkt(new_license_request, license_info: true)
+
+    # Expect that we are issued a license here
+    res = rdp_send_recv(pkt)
+    rdp_parse_license_pdu(res)
+  end
+
+  def rdp_handle_license_error_alert(data)
+    error_code, state_transition, error_info = data[0..11].unpack("VVV")
+    vprint_status("License error/alert code 0x#{error_code.to_s(16)} (#{RDPConstants::LICENSE_ERRS[error_code]})")
+    # Ensure that we were issued a license by the server
+    raise RdpCommunicationError if error_code != RDPConstants::LICENSE_ERR_LICENSE_ISSUED
+  end
+
+  def rdp_parse_license_pdu(data)
+    raise RdpCommunicationError if data.length < 20
+    rdp_version = data[0].unpack("C")[0]
+    raise RdpCommunicationError if rdp_version != 3
+    length = data[2..3].unpack("n")[0]
+    raise RdpCommunicationError if data.length != length
+
+    data_len = data[13].unpack("C")[0]
+    tag_offset = 18
+    tag_offset += 1 if (data_len & 0x80 == 0x80) # 2 byte length
+
+    tag = data[tag_offset].unpack("C")[0]
+    vprint_status("Got license packet type 0x#{tag.to_s(16)} (#{RDPConstants::LICENSE_TAGS[tag]})")
+
+    case tag
+      when RDPConstants::LICENSE_REQUEST
+        rdp_handle_license_request(data[tag_offset + 4..-1])
+      when RDPConstants::LICENSE_PLATFORM_CHALLENGE
+      when RDPConstants::LICENSE_NEW_LICENSE
+      when RDPConstants::LICENSE_UPGRADE_LICENSE
+      when RDPConstants::LICENSE_LICENSE_INFO
+      when RDPConstants::LICENSE_NEW_LICENSE_REQ
+      when RDPConstants::LICENSE_PLATFORM_CHAL_RESP
+      when RDPConstants::LICENSE_ERROR_ALERT
+        rdp_handle_license_error_alert(data[tag_offset + 4..-1])
+    end
+  end
+
   # Finish building session after all security is negotiated
   def rdp_establish_session
     vprint_status("Sending client info PDU")
-    res = rdp_send_recv(rdp_build_pkt(pdu_client_info(@user_name, @domain, @ip_address), "\x03\xeb", true))
-    vprint_status("Received License packet")
+    res = rdp_send_recv(rdp_build_pkt(pdu_client_info(@user_name, @domain, @ip_address),
+                                      "\x03\xeb", client_info: true))
+    vprint_status("Received License packet (#{res.length} bytes)")
+    rdp_parse_license_pdu(res)
 
     # Windows XP sometimes sends a very large license packet. This is likely
     # some form of license error. When it does this it doesn't send a Server
@@ -396,6 +576,16 @@ module Exploit::Remote::RDP
     [md5.hexdigest].pack("H*")
   end
 
+  def rdp_salted_hash16(s_bytes, salt1, salt2)
+    md5 = Digest::MD5.new
+
+    md5 << s_bytes[0..15]
+    md5 << salt1[0..31]
+    md5 << salt2[0..31]
+
+    [md5.hexdigest].pack("H*")
+  end
+
   # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpbcgr/705f9542-b0e3-48be-b9a5-cf2ee582607f
   #  SaltedHash(S, I) = MD5(S + SHA(I + S + ClientRandom + ServerRandom))
   def rdp_salted_hash(s_bytes, i_bytes, client_random_bytes, server_random_bytes)
@@ -413,6 +603,12 @@ module Exploit::Remote::RDP
     [md5.hexdigest].pack("H*")
   end
 
+  def rdp_salted_hash48(s_bytes, i_byte, client_random, server_random)
+    rdp_salted_hash(s_bytes, i_byte, client_random, server_random) + \
+      rdp_salted_hash(s_bytes, (i_byte.ord + 1).chr * 2, client_random, server_random) + \
+      rdp_salted_hash(s_bytes, (i_byte.ord + 2).chr * 3, client_random, server_random)
+  end
+
   #  FinalHash(K) = MD5(K + ClientRandom + ServerRandom)
   def rdp_final_hash(k, client_random_bytes, server_random_bytes)
     md5 = Digest::MD5.new
@@ -425,16 +621,16 @@ module Exploit::Remote::RDP
   end
 
   def rdp_calculate_rc4_keys(client_random, server_random)
-    # preMasterSecret = First192Bits(ClientRandom) + First192Bits(ServerRandom)
-    preMasterSecret = client_random[0..23] + server_random[0..23]
+    # g = First192Bits(ClientRandom) + First192Bits(ServerRandom)
+    g = client_random[0..23] + server_random[0..23]
 
-    # PreMasterHash(I) = SaltedHash(preMasterSecret, I)
+    # PreMasterHash(I) = SaltedHash(g, I)
     # MasterSecret = PreMasterHash(0x41) + PreMasterHash(0x4242) + PreMasterHash(0x434343)
-    masterSecret = rdp_salted_hash(preMasterSecret, "A", client_random,server_random) +  rdp_salted_hash(preMasterSecret, "BB", client_random, server_random) + rdp_salted_hash(preMasterSecret, "CCC", client_random, server_random)
+    master_secret = rdp_salted_hash48(g, "A", client_random, server_random)
 
     # MasterHash(I) = SaltedHash(MasterSecret, I)
     # SessionKeyBlob = MasterHash(0x58) + MasterHash(0x5959) + MasterHash(0x5A5A5A)
-    sessionKeyBlob = rdp_salted_hash(masterSecret, "X", client_random, server_random) +  rdp_salted_hash(masterSecret, "YY", client_random, server_random) + rdp_salted_hash(masterSecret, "ZZZ", client_random, server_random)
+    sessionKeyBlob = rdp_salted_hash48(master_secret, "X", client_random, server_random)
 
     # InitialClientDecryptKey128 = FinalHash(Second128Bits(SessionKeyBlob))
     initialClientDecryptKey128 = rdp_final_hash(sessionKeyBlob[16..31], client_random, server_random)
@@ -491,10 +687,11 @@ module Exploit::Remote::RDP
 
   # Build the X.224 packet, encrypt with Standard RDP Security as needed
   # default channel_id = 0x03eb = 1003
-  def rdp_build_pkt(data, channel_id = "\x03\xeb", client_info = false)
+  def rdp_build_pkt(data, channel_id = "\x03\xeb", client_info: false, license_info: false)
     flags = 0
-    flags |= 0b1000 if @rdp_sec       # Set SEC_ENCRYPT
-    flags |= 0b1000000 if client_info # Set SEC_INFO_PKT
+    flags |= 0x08 if @rdp_sec     # Set SEC_ENCRYPT
+    flags |= 0x40 if client_info  # Set SEC_INFO_PKT
+    flags |= 0x80 if license_info # Set SEC_LICENSE_PKT
 
     pdu = ""
 
@@ -546,6 +743,28 @@ module Exploit::Remote::RDP
       data
   end
 
+  # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpele/c57e4890-9049-421e-9fe8-9a6f9519675a
+  # Client New License Request PDU - 2.2.2.2
+  def pdu_new_license_request(client_random, user, host)
+    length = 24 + client_random.length + 64 + user.length + 1 + host.length + 1
+
+    [RDPConstants::LICENSE_NEW_LICENSE_REQ].pack("C") +
+    "\x03" +                         # Version
+    [length].pack("S<") +            # Length
+    "\x01\x01\x00\x00\x00\x01\xff" + # KEY_EXCHANGE_ALG_RSA
+    client_random[0..31] +
+    "\x02\x00" +                     # Encrypted Premaster Secret RANDOM_BLOB
+    [64].pack("S<") +
+    "\x00" * 64 +                    # The client license premaster secret, we don't care about the license contents
+    "\x0f\x00" +                     # USER_NAME_BLOB
+    [user.length + 1].pack("S<") +
+    user + "\x00" +
+    "\x10\x00" +                     # CLIENT_MACHINE_NAME_BLOB
+    [host.length + 1].pack("S<") +
+    host + "\x00"
+
+  end
+
   # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpbcgr/18a27ef9-6f9a-4501-b000-94b1fe3c2c10
   # Client X.224 Connect Request PDU - 2.2.1.1
   def pdu_negotiation_request(user_name = "", requested_protocols = 0)
@@ -571,125 +790,51 @@ module Exploit::Remote::RDP
   end
 
   # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpbcgr/db6713ee-1c0e-4064-a3b3-0fac30b4037b
-  def pdu_connect_initial(selected_proto = 0, host_name = "rdesktop")
+  def pdu_connect_initial(channels, selected_proto = 0, host_name = "rdesktop")
     # After negotiating TLS or NLA the connectInitial packet needs to include the
     # protocol selection that the server indicated in its Negotiation Response
 
-    # TODO: If this is pulled into an RDP library then the channel list likely
-    # needs to be build dynamically. For example, MS_T120 likely should only
-    # ever be sent as part of checks for CVE-2019-0708
+    pdu = [
+      "\x7f\x65",        # T.125 Connect-Initial (BER: Application 101)
+      ber_data(
+        "\x04\x01\x01",    # CallingDomainSelector: 1 (BER: OctetString)
+        "\x04\x01\x01",    # CalledDomainSelector: 1 (BER: OctetString)
+        "\x01\x01\xff",    # UpwaredFlag: True (BER: boolean)
 
-    # build clientName - 12.2.1.3.2 Client Core Data (TS_UD_CS_CORE)
-    # 15 characters + null terminator, converted to unicode
-    # fixed length - 32 characters total
-    name_unicode = Rex::Text.to_unicode(host_name[0..14], 'utf-16le')
-    name_unicode += "\x00" * (32 - name_unicode.length)
-
-    pdu =
-      "\x7f\x65" +       # T.125 Connect-Initial (BER: Application 101)
-      "\x82\x01\xbe" +   # Length (BER: Length)
-      "\x04\x01\x01" +   # CallingDomainSelector: 1 (BER: OctetString)
-      "\x04\x01\x01" +   # CalledDomainSelector: 1 (BER: OctetString)
-      "\x01\x01\xff" +   # UpwaredFlag: True (BER: boolean)
-      # Connect-Initial: Target Parameters
-      "\x30\x20" +       # TargetParamenters (BER: SequenceOf)
-      # Not sure why the BER encoded Integers below have 2 byte values instead of one.
-      "\x02\x02\x00\x22" +  # MaxChannelIds: 34    (BER: Int)
-      "\x02\x02\x00\x02" +  # MaxUserIDs: 2        (BER: Int)
-      "\x02\x02\x00\x00" +  # MaxTokenIds: 0       (BER: Int)
-      "\x02\x02\x00\x01" +  # NumPriorities: 1     (BER: Int)
-      "\x02\x02\x00\x00" +  # MinThroughput: 0     (BER: Int)
-      "\x02\x02\x00\x01" +  # MaxHeight: 1         (BER: Int)
-      "\x02\x02\xff\xff" +  # MaxMCSPDUSize: 65535 (BER: Int)
-      "\x02\x02\x00\x02" +  # ProtocolVersion: 2   (BER: Int)
-      # Connect-Intial: Minimum Parameters
-      "\x30\x20" +          # MinimumParameters (BER: SequencOf)
-      "\x02\x02\x00\x01" +  # MaxChannelIds: 1     (BER: Int)
-      "\x02\x02\x00\x01" +  # MaxUserIDs: 1        (BER: Int)
-      "\x02\x02\x00\x01" +  # MaxTokenIds: 1       (BER: Int)
-      "\x02\x02\x00\x01" +  # NumPriorities: 1     (BER: Int)
-      "\x02\x02\x00\x00" +  # MinThroughput: 0     (BER: Int)
-      "\x02\x02\x00\x01" +  # MaxHeight: 1         (BER: Int)
-      "\x02\x02\x04\x20" +  # MaxMCSPDUSize: 1056  (BER: Int)
-      "\x02\x02\x00\x02" +  # ProtocolVersion: 2   (BER: Int)
-      # Connect-Initial: Maximum Parameters
-      "\x30\x20" +          # MaximumParameters (BER: SequencOf)
-      "\x02\x02\xff\xff" +  # MaxChannelIds: 65535  (BER: Int)
-      "\x02\x02\xfc\x17" +  # MaxUserIDs: 64535     (BER: Int)
-      "\x02\x02\xff\xff" +  # MaxTokenIds: 65535    (BER: Int)
-      "\x02\x02\x00\x01" +  # NumPriorities: 1      (BER: Int)
-      "\x02\x02\x00\x00" +  # MinThroughput: 0      (BER: Int)
-      "\x02\x02\x00\x01" +  # MaxHeight: 1          (BER: Int)
-      "\x02\x02\xff\xff" +  # MaxMCSPDUSize: 65535  (BER: Int)
-      "\x02\x02\x00\x02" +  # ProtocolVersion: 2    (BER: Int)
-      # Connect-Initial: UserData
-      "\x04\x82\x01\x4b" +  # UserData, length 331  (BER: OctetString)
-      # T.124 GCC Connection Data (ConnectData)- PER Encoding used
-      "\x00\x05" + # object length
-      "\x00\x14\x7c\x00\x01" + # object: OID 0.0.20.124.0.1 = Generic Conference Control
-      "\x81\x42" + # Length: 322 (Connect PDU)
-      "\x00\x08\x00\x10\x00\x01\xc0\x00" + # T.124 Connect PDU, Conference name 1
-      "\x44\x75\x63\x61" + # h221NonStandard: 'Duca' (client-to-server H.221 key)
-      "\x81\x34" + # Length: 308 (T.124 UserData section)
-      # Client MCS Section - 2.2.1.3
-      "\x01\xc0" + # clientCoreData (TS_UD_CS_CORE) header - 2.2.1.3.2
-      "\xd8\x00" + # Length: 216 (includes header)
-      "\x04\x00\x08\x00" + # version: 8.4 (RDP 5.0 -> 8.1)
-      "\x20\x03" + # desktopWidth: 800
-      "\x58\x02" + # desktopHeigth: 600
-      "\x01\xca" + # colorDepth: 8 bpp
-      "\x03\xaa" + # SASSequence: 43523
-      "\x09\x04\x00\x00" + # keyboardLayout: 1033 (English US)
-      "\x28\x0a\x00\x00" + # clientBuild: 2600
-      [name_unicode].pack("a*") + # clientName
-      "\x04\x00\x00\x00" + # keyboardType: 4 (IBMEnhanced 101 or 102)
-      "\x00\x00\x00\x00" + # keyboadSubtype: 0
-      "\x0c\x00\x00\x00" + # keyboardFunctionKey: 12
-      # imeFileName (64 bytes)
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x01\xca" + # postBeta2ColorDepth: 8 bpp
-      "\x01\x00" + # clientProductID: 1
-      "\x00\x00\x00\x00" + # serialNumber: 0
-      "\x18\x00" + # highColorDepth: 24 bpp
-      "\x07\x00" + # supportedColorDepths: flag (24 bpp | 16 bpp | 15 bpp )
-      "\x01\x00" + # earlyCapabilityFlags: 1 (RNS_UD_CS_SUPPORT_ERRINFO_PDU)
-      # clientDigProductID (64 bytes)
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" + #
-      "\x00" + # connectionType: 0
-      "\x00" + # pad1octet
-      # serverSelectedProtocol - After negotiating TLS or CredSSP this value must
-      # match the selectedProtocol value from the server's Negotiate Connection
-      # confirm PDU that was sent before encryption was started.
-      [selected_proto].pack('L<') +
-      "\x04\xc0" + # clientClusterdata (TS_UD_CS_CLUSTER) header - 2.2.1.3.5
-      "\x0c\x00" + # Length: 12 (includes header)
-      "\x09\x00\x00\x00" + # flags (REDIRECTION_SUPPORTED | REDIRECTION_VERSION3)
-      "\x00\x00\x00\x00" + # RedirectedSessionID
-      "\x02\xc0" + # clientSecuritydata (TS_UD_CS_SEC) header - 2.2.1.3.3
-      "\x0c\x00" + # Length: 12 (includes header)
-      "\x03\x00\x00\x00" + # encryptionMethods: 3 (40 bit | 128 bit)
-      "\x00\x00\x00\x00" + # extEncryptionMethods (French locale only)
-      "\x03\xc0" + # clientNetworkData (TS_UD_CS_NET) - 2.2.1.3.4
-      "\x44\x00" + # Length: 68 (includes header)
-      "\x05\x00\x00\x00" + # channelCount: 5
-      # Channels definitions consist of a name (8 bytes) and options flags
-      # (4 bytes). Names are up to 7 ANSI characters with null termination.
-      "\x63\x6c\x69\x70\x72\x64\x72\x00" + # 'cliprdr'
-      "\xc0\xa0\x00\x00" + #
-      "\x4d\x53\x5f\x54\x31\x32\x30\x00" + # 'MS_T120'
-      "\x80\x80\x00\x00" + #
-      "\x72\x64\x70\x73\x6e\x64\x00\x00" + # 'rdpsnd
-      "\xc0\x00\x00\x00" + #
-      "\x73\x6e\x64\x64\x62\x67\x00\x00" + # 'snddbg'
-      "\xc0\x00\x00\x00" + #
-      "\x72\x64\x70\x64\x72\x00\x00\x00" + # 'rdpdr'
-      "\x80\x80\x00\x00"
+        # TargetParamenters
+        encode_domain_selector(
+          max_chan_ids: 0x22,
+          max_user_ids: 0x2
+        ),
+        # MinimumParameters
+        encode_domain_selector(
+          max_chan_ids: 0x1,
+          max_user_ids: 0x1,
+          max_token_ids: 0x1,
+          max_mcspdu_size: 0x0420
+        ),
+        # MaximumParameters
+        encode_domain_selector(
+          max_chan_ids: 0xffff,
+          max_user_ids: 0xfc17,
+          max_token_ids: 0xffff
+        ),
+        # UserData
+        ber_octet_string(
+          # T.124 GCC Connection Data (ConnectData)- PER Encoding used
+          per_object(oid(0, 0, 20, 124, 0, 1)),
+          per_data(
+            conf_create_req(),
+            per_data(
+              cs_core_data(client_name: host_name, selected_proto: selected_proto),
+              cs_cluster_data(),
+              cs_security_data(),
+              cs_network_data(channels)
+            )
+          )
+        )
+      )
+    ].join('')
 
     build_data_tpdu(pdu)
   end
@@ -715,7 +860,7 @@ module Exploit::Remote::RDP
 
   # https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpbcgr/64564639-3b2d-4d2c-ae77-1105b4cc011b
   # Client MCS Channel Join Request PDU -2.2.1.8
-  def pdu_channel_request(user1, channel_id)
+  def pdu_channel_join_request(user1, channel_id)
     pdu =
       "\x38" + # T.125 ChannelJoinRequest
       [user1, channel_id].pack("nn")
@@ -1005,16 +1150,331 @@ module Exploit::Remote::RDP
 
   # Create a new SSL session on the existing socket.
   # Stolen from exploit/smtp_deliver.rb
-  def swap_sock_plain_to_ssl(nsock)
+  def swap_sock_plain_to_ssl
     ctx = OpenSSL::SSL::SSLContext.new
     ctx.min_version = OpenSSL::SSL::TLS1_VERSION
-    ssl = OpenSSL::SSL::SSLSocket.new(nsock, ctx)
+    ctx.security_level = datastore['RDP_TLS_SECURITY_LEVEL']
+    ssl = OpenSSL::SSL::SSLSocket.new(self.rdp_sock, ctx)
 
-    ssl.connect
+    begin
+      ssl.connect
+    rescue Errno::ECONNRESET
+      vprint_error("Retry with advanced option RDP_TLS_SECURITY_LEVEL=0")
+      raise
+    end
 
-    nsock.extend(Rex::Socket::SslTcp)
-    nsock.sslsock = ssl
-    nsock.sslctx  = ctx
+    self.rdp_sock.extend(Rex::Socket::SslTcp)
+    self.rdp_sock.sslsock = ssl
+    self.rdp_sock.sslctx  = ctx
   end
+
+protected
+
+  def encode_domain_selector(
+    max_chan_ids: 0,
+    max_user_ids: 0,
+    max_token_ids: 0,
+    num_priorities: 1,
+    min_throughput: 0,
+    max_height: 1,
+    max_mcspdu_size: 65535,
+    protocol_ver: 2
+  )
+
+    body = [
+      ber_int(max_chan_ids),
+      ber_int(max_user_ids),
+      ber_int(max_token_ids),
+      ber_int(num_priorities),
+      ber_int(min_throughput),
+      ber_int(max_height),
+      ber_int(max_mcspdu_size),
+      ber_int(protocol_ver)
+    ].join('')
+
+    result = [
+      "\x30",
+      [body.length].pack('C'),
+      body
+    ].join('')
+
+    result
+  end
+
+  def per_object(*ds)
+    body = ds.join('')
+
+    result = [
+      "\x00",
+      [body.length].pack('C'),
+      body
+    ].join('')
+
+    result
+  end
+
+  def per_data(*ds)
+    data = ds.join('')
+    result = ''
+    if data.length < 0x4000
+      result = [data.length | 0x8000].pack('S>') + data
+    else
+      result = "\xA2" + [data.length].pack('S>') + data
+    end
+
+    result
+  end
+
+  def cs_cluster_data(
+    flags: RDPConstants::REDIRECTION_SUPPORTED | RDPConstants::REDIRECTION_VERSION3,
+    session_id: 0
+  )
+    body = [flags, session_id].pack('<L<L')
+
+    result = [
+      [0xc004, body.length + 4].pack('<S<S'),
+      body
+    ].join('')
+
+    result
+  end
+
+  def cs_security_data(
+    encryption_methods: RDPConstants::ENCRYPTION_40BIT | RDPConstants::ENCRYPTION_128BIT,
+    ext_encryption_methods: 0
+  )
+    body = [encryption_methods, ext_encryption_methods].pack('<L<L')
+
+    result = [
+      [0xc002, body.length + 4].pack('<S<S'),
+      body
+    ].join('')
+
+    result
+  end
+
+  def cs_network_data(channels)
+    chan_data = channels.map{ |c|
+      [c[0].encode('ASCII')].pack('a8*') + [c[1]].pack('L')
+    }.join('')
+
+    body = [
+      [channels.length].pack('L'),
+      chan_data
+    ].join('')
+
+    result = [
+      [0xc003, body.length + 4].pack('<S<S'),
+      body
+    ].join('')
+
+    result
+  end
+
+
+  def cs_core_data(
+    version: 0x80004,
+    width: 800,
+    height: 600,
+    keyboard: 1033, # English
+    client_build: 2600,
+    client_name: "rdesktop",
+    keyboard_type: 4, # IBMEhanced 101/102
+    keyboard_subtype: 0,
+    keyboard_func_key: 12,
+    serial_num: 0,
+    client_product_id: 1,
+    client_dig_product_id: "",
+    selected_proto: 0
+  )
+
+    client_name = Rex::Text.to_unicode(client_name[0..16], 'utf-16le')
+    client_dig_product_id = Rex::Text.to_unicode(client_dig_product_id[0..32], 'utf-16le')
+
+    body = [
+      [version, width, height].pack('<L<S<S'),
+      "\x01\xca", # colour depth (8BPP)
+      "\x03\xaa", # SASSequence
+      [keyboard, client_build, client_name, keyboard_type].pack('<L<La32*'),
+      [keyboard_type, keyboard_subtype, keyboard_func_key].pack('<L<L<L'),
+      "\x00" * 64, # imeFileName
+      "\x01\xca", # postBeta2ColorDepth (8BPP)
+      [client_product_id, serial_num].pack('<S<L'),
+      "\x18\x00", # highColorDepth: 24 bpp
+      "\x07\x00", # supportedColorDepths: flag (24 bpp | 16 bpp | 15 bpp )
+      "\x01\x00", # earlyCapabilityFlags: 1 (RNS_UD_CS_SUPPORT_ERRINFO_PDU)
+      [client_dig_product_id].pack('a64*'),
+      "\x00", # connectionType: 0
+      "\x00", # pad1octet
+      # serverSelectedProtocol - After negotiating TLS or CredSSP this value must
+      # match the selectedProtocol value from the server's Negotiate Connection
+      # confirm PDU that was sent before encryption was started.
+      [selected_proto].pack('L<')
+    ].join('')
+
+    result = [
+      [0xc001, body.length + 4].pack('<S<S'),
+      body
+    ].join('')
+
+    result
+  end
+
+  def conf_create_req(user_data_sets: 1, h221_key: "Duca")
+    b2 = 0
+    b2 |= 0x08 if user_data_sets > 0
+
+    b5 = 0x40
+    b5 |= 0x80 if user_data_sets > 0
+
+    # TODO: add more flags here
+    [
+      "\x00",
+      [b2].pack('C'),
+      "\x00\x10\x00",
+      [user_data_sets].pack('C'),
+      [b5].pack('C'),
+      "\x00",
+      [h221_key.encode('ASCII')].pack('a*')
+    ].join('')
+  end
+
+  def oid(itut, rec, t, t124, ver, desc)
+    [(itut << 8) | rec, t, t124, ver, desc].pack('C*')
+  end
+
+  def ber_octet_string(*ds)
+    result = [
+      "\x04",
+      ber_data(ds)
+    ].join('')
+
+    result
+  end
+
+  def ber_data(*ds)
+    data = ds.join('')
+
+    result = [
+      "\x82",
+      [data.length].pack('S>'),
+      data
+    ].join('')
+
+    result
+  end
+
+  def ber_int(i)
+    d = ''
+    if i < (2 ** 8)
+      d = [i].pack('C')
+    elsif i < (2 ** 16)
+      d = [i].pack('S>')
+    else
+      d = [i].pack('L>')
+    end
+
+    "\x02" + [d.length].pack('C') + d
+  end
+
+  def rdp_handle_packet(pkt)
+    if pkt && pkt[0] == "\x03"
+      if pkt[4..6] == "\x02\xf0\x80"
+        if pkt[7] == "\x68"
+          chan_user_id = pkt[8..9].unpack('S>')[0]
+          chan_id = pkt[10..11].unpack('S>')[0]
+          flags = pkt[18..21].unpack('<L')[0]
+          data = pkt[22..pkt.length]
+          rdp_on_channel_receive(pkt, chan_user_id, chan_id, flags, data)
+        end
+      end
+    end
+  end
+
+  def rdp_on_channel_receive(pkt, chan_user_id, chan_id, flags, data)
+    ctype = data[0..1].unpack('S')[0]
+
+    if ctype == RDPConstants::RDPDR_CTYP_CORE
+      opcode = data[2..3].unpack('S')[0]
+      if opcode == RDPConstants::PAKID_CORE_SERVER_ANNOUNCE
+        rdp_on_core_server_announce(pkt, chan_user_id, chan_id, flags, data)
+      elsif opcode == RDPConstants::PAKID_CORE_SERVER_CAPABILITY
+        rdp_on_core_server_capability(pkt, chan_user_id, chan_id, flags, data)
+      elsif opcode == RDPConstants::PAKID_CORE_CLIENTID_CONFIRM
+        rdp_on_core_client_id_confirm(pkt, chan_user_id, chan_id, flags, data)
+      end
+    end
+  end
+
+  def rdp_on_core_server_announce(pkt, chan_user_id, chan_id, flags, data)
+    vprint_status("Handling SERVER ANNOUNCE ...")
+    rdpdr_client_announce_reply(pkt, chan_user_id, chan_id, flags, data)
+    rdpdr_client_name_request(pkt, chan_user_id, chan_id, flags, data)
+  end
+
+  def rdp_on_core_server_capability(pkt, chan_user_id, chan_id, flags, data)
+    vprint_status("Handling SERVER CAPABILITY ...")
+    # change opcode 1 byte to match server capabilities
+    reply = [data[0..2], "\x43", data[4..data.length]].join('')
+
+    rdp_send_channel(chan_user_id, chan_id, reply)
+  end
+
+  def rdp_on_core_client_id_confirm(pkt, chan_user_id, chan_id, flags, data)
+    vprint_status("Handling CLIENT ID CONFIRM ...")
+    rdpdr_client_device_list_announce_request(pkt, chan_user_id, chan_id, flags, data)
+  end
+
+  def rdpdr_client_device_list_announce_request(pkt, chan_user_id, chan_id, flags, data)
+    reply = [
+      RDPConstants::RDPDR_CTYP_CORE,
+      RDPConstants::PAKID_CORE_DEVICELIST_ANNOUNCE,
+      0x0, # Device count
+    ].pack('SSL')
+
+    rdp_send_channel(chan_user_id, chan_id, reply)
+  end
+
+  def rdpdr_client_announce_reply(pkt, chan_user_id, chan_id, flags, data)
+    reply = [
+      RDPConstants::RDPDR_CTYP_CORE,
+      RDPConstants::PAKID_CORE_CLIENTID_CONFIRM,
+      0x1, # Version Major
+      0xc, # Version Minor
+      0x2, # client ID (TODO: configure this? read it from the packet?
+    ].pack('SSSSL')
+
+    rdp_send_channel(chan_user_id, chan_id, reply)
+  end
+
+  def rdpdr_client_name_request(pkt, chan_user_id, chan_id, flags, data)
+    computer_name = Rex::Text.to_unicode("#{@computer_name}\x00", 'utf-16le')
+    reply = [
+      RDPConstants::RDPDR_CTYP_CORE,
+      RDPConstants::PAKID_CORE_CLIENT_NAME,
+      0x1, # Unicode flag
+      0x0, # Code Page
+      computer_name.length,
+      computer_name,
+    ].pack('SSLLLa*')
+
+    rdp_send_channel(chan_user_id, chan_id, reply)
+  end
+
+  attr_accessor :rdp_sock
+
+  attr_accessor :rdp_user_id
+
+=begin
+  # debug stuff
+  def rdp_to_file(b, del = false)
+    p = "/tmp/ruby-full.bin"
+    ::File.delete(p) if del && ::File.exist?(p)
+    f = ::File.new(p, "ab")
+    f.write(b)
+    f.close
+  end
+=end
+
 end
 end
