@@ -171,6 +171,14 @@ module Exploit::Remote::Tcp
     return rv
   end
 
+  #
+  # Shutdown the TCP connection
+  #
+  def shutdown(how = :SHUT_RDWR)
+    self.sock.shutdown(how) if self.sock
+  rescue IOError
+  end
+
   #
   # Closes the TCP connection
   #
