@@ -471,7 +471,7 @@ class Creds
         private_val = core.private ? core.private.to_s : ""
         realm_val = core.realm ? core.realm.value : ""
         human_val = core.private ? core.private.class.model_name.human : ""
-        jtr_val = core.private.jtr_format ? core.private.jtr_format : ""
+        jtr_val = core.private ? core.private.jtr_format : ""
 
         tbl << [
           "", # host
