@@ -60,6 +60,7 @@ class Console::CommandDispatcher::Core
       'use'                      => 'Deprecated alias for "load"',
       'load'                     => 'Load one or more meterpreter extensions',
       'machine_id'               => 'Get the MSF ID of the machine attached to the session',
+      'secure'                   => '(Re)Negotiate TLV packet encryption on the session',
       'guid'                     => 'Get the session GUID',
       'quit'                     => 'Terminate the meterpreter session',
       'resource'                 => 'Run the commands stored in a file',
@@ -319,6 +320,12 @@ class Console::CommandDispatcher::Core
     end
   end
 
+  def cmd_secure
+    print_status('Negotiating new encryption key ...')
+    client.core.secure
+    print_good('Done.')
+  end
+
   def cmd_background_help
     print_line('Usage: background')
     print_line
