@@ -39,9 +39,7 @@ class MetasploitModule < Msf::Exploit::Local
       ],
       'DefaultTarget' => 0,
       'References'    => [
-        [
-          'URL', 'https://github.com/L3cr0f/DccwBypassUAC'
-        ]
+        ['URL', 'https://github.com/L3cr0f/DccwBypassUAC']
       ],
       'DisclosureDate'=> 'Apr 06 2017'
     ))
